@@ -6,7 +6,7 @@
 #include <stdio.h>
 #include <glib.h>
 
-#include "rsvg.h"
+#include "librsvg/rsvg.h"
 #include "test-utils.h"
 
 /* These tests are meant to test the error handlers in librsvg.  As of 2.44.x we
@@ -96,5 +96,6 @@ main (int argc, char **argv)
                                test_loading_error,
                                NULL);
 
+
     return g_test_run ();
 }
